package com.example.user.myapp;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.support.v4.content.LocalBroadcastManager;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.ToggleButton;

import java.util.HashMap;
import java.util.Map;

public class ManualDriveActivity extends AppCompatActivity {
    private GlobalState state;
    private Handler coilHandler;

    private HashMap<Integer, Boolean> mapCoilCheckStatus = new HashMap <> ();
    private HashMap<ToggleButton, Integer> mapButtonCoil = new HashMap<>();
    private boolean manualMode = false;

    private void initMap () {
        mapCoilCheckStatus.put(64, false);
        mapCoilCheckStatus.put(70, false);
        mapCoilCheckStatus.put(65, false);
        mapCoilCheckStatus.put(71, false);
        mapCoilCheckStatus.put(68, false);
        mapCoilCheckStatus.put(69, false);
        mapCoilCheckStatus.put(66, false);
        mapCoilCheckStatus.put(67, false);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        state = (GlobalState) getApplicationContext();
        initMap(); // initialise the value of the coil toggle state + the corresponding coil to button

        setContentView(R.layout.activity_manual_drive);

        ToggleButton h1ControlButton = (ToggleButton) findViewById(R.id.h1_control_button);
        ToggleButton h2ControlButton = (ToggleButton) findViewById(R.id.h2_control_button);
        Button pu1ControlButton = (Button) findViewById(R.id.pu1_control_button);
        Button pu2ControlButton = (Button) findViewById(R.id.pu2_control_button);
        Button sp1ControlButton = (Button) findViewById(R.id.sp1_control_button);
        Button b1ControlButton = (Button) findViewById(R.id.b1_control_button);
        ToggleButton m1ControlButton = (ToggleButton) findViewById(R.id.m1_control_button);
        ToggleButton m2ControlButton = (ToggleButton) findViewById(R.id.m2_control_button);

        mapButtonCoil.put(h1ControlButton, 65);
        mapButtonCoil.put(h2ControlButton, 70);
        mapButtonCoil.put(m1ControlButton, 66);
        mapButtonCoil.put(m2ControlButton, 67);

        LocalBroadcastManager.getInstance(this).registerReceiver(coilWriterMessageReceiver, new IntentFilter("readVmCoil2"));
     //   LocalBroadcastManager.getInstance(this).registerReceiver(coilWriterMessageReceiver, new IntentFilter("checkCoilValue"));

        // manual drive button
       ToggleButton manualDriveOverrideButton = (ToggleButton) findViewById(R.id.manual_override_button);
        manualDriveOverrideButton.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    // writing the value true
                    Intent serviceIntent = new Intent(getApplicationContext(), MyIntentService.class);
                    serviceIntent.setAction("write.coil");
                    serviceIntent.putExtra("extra.ip.address", state.getIpAddress());
                    serviceIntent.putExtra("extra.ip.port", state.getPort());
                    serviceIntent.putExtra("extra.ref", GlobalState.InputConfig.VM_COIL2.getStartReference());
                    serviceIntent.putExtra("extra.bit", true);
                    serviceIntent.putExtra("extra.intent.name", "readVmCoil2");
                    serviceIntent.putExtra("extra.is.basic.coil", false);
                    getApplicationContext().startService(serviceIntent);
                } else {
                    // writing the value false
                    Intent serviceIntent = new Intent(getApplicationContext(), MyIntentService.class);
                    serviceIntent.setAction("write.coil");
                    serviceIntent.putExtra("extra.ip.address", state.getIpAddress());
                    serviceIntent.putExtra("extra.ip.port", state.getPort());
                    serviceIntent.putExtra("extra.ref", GlobalState.InputConfig.VM_COIL2.getStartReference());
                    serviceIntent.putExtra("extra.bit", false);
                    serviceIntent.putExtra("extra.intent.name", "readVmCoil2");
                    serviceIntent.putExtra("extra.is.basic.coil", false);
                    getApplicationContext().startService(serviceIntent);
                }
            }
        });

        // for each toggle button, set the event
        for (Map.Entry<ToggleButton, Integer> entry : mapButtonCoil.entrySet()) {
            final ToggleButton button = entry.getKey();
            final Integer coilToWrite = entry.getValue();
            button.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                    if (isChecked) {
                        // writing the value true
                        Intent serviceIntent = new Intent(getApplicationContext(), MyIntentService.class);
                        serviceIntent.setAction("write.coil");
                        serviceIntent.putExtra("extra.ip.address", state.getIpAddress());
                        serviceIntent.putExtra("extra.ip.port", state.getPort());
                        serviceIntent.putExtra("extra.ref", coilToWrite);
                        serviceIntent.putExtra("extra.bit", true);
                        serviceIntent.putExtra("extra.intent.name", "writeBit65");
                        serviceIntent.putExtra("extra.is.basic.coil", true);
                        getApplicationContext().startService(serviceIntent);
                        mapCoilCheckStatus.put(coilToWrite, true);
                    } else {
                        // writing the value false
                        Intent serviceIntent = new Intent(getApplicationContext(), MyIntentService.class);
                        serviceIntent.setAction("write.coil");
                        serviceIntent.putExtra("extra.ip.address", state.getIpAddress());
                        serviceIntent.putExtra("extra.ip.port", state.getPort());
                        serviceIntent.putExtra("extra.ref", coilToWrite);
                        serviceIntent.putExtra("extra.bit", false);
                        serviceIntent.putExtra("extra.intent.name", "writeBit65");
                        serviceIntent.putExtra("extra.is.basic.coil", true);
                        getApplicationContext().startService(serviceIntent);
                        mapCoilCheckStatus.put(coilToWrite, false);
                    }
                }
            });
        }
    }

    @Override
    protected void onStart () {
        super.onStart();
        coilHandler = new Handler();
        coilHandler.post(refreshCoil);
    }

    private BroadcastReceiver coilWriterMessageReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            boolean supposedToBeValue = intent.getBooleanExtra("coil", false);

            ImageView lightOn = (ImageView) findViewById(R.id.manual_override_status_light_on);
            ImageView lightOff = (ImageView) findViewById(R.id.manual_override_status_light_on);

            System.out.print("value "+supposedToBeValue);
            if (supposedToBeValue) {
                manualMode = true;
                lightOn.setVisibility(View.VISIBLE);
                lightOff.setVisibility(View.INVISIBLE);
            } else {
                manualMode = false;
                lightOn.setVisibility(View.INVISIBLE);
                lightOff.setVisibility(View.VISIBLE);
            }
        }
    };

    Runnable refreshCoil = new Runnable() {
        @Override
        public void run() {
            System.out.println("pass "+manualMode);
            if (manualMode) {
                for (Map.Entry<Integer, Boolean> entry : mapCoilCheckStatus.entrySet()) {
                    System.out.println("entry : " + entry.getKey() + " value :" + entry.getValue());
                    if (entry.getValue() == true) { // if the thing is toggle
                        Intent serviceIntent = new Intent(getApplicationContext(), MyIntentService.class);
                        serviceIntent.setAction("read.coil");
                        serviceIntent.putExtra("extra.ip.address", state.getIpAddress());
                        serviceIntent.putExtra("extra.ip.port", state.getPort());
                        serviceIntent.putExtra("extra.ref", state.getMapMappingValue().get(entry.getKey()));
                        serviceIntent.putExtra("extra.count", 1);
                        serviceIntent.putExtra("extra.intent.name", "read");
                        getApplicationContext().startService(serviceIntent);
                    }
                }
            }
            coilHandler.postDelayed(this, 2000);
        }
    };

}
